#ifndef ___GZIP_H_
#define ___GZIP_H_

/* gzip.h -- common declarations for all libgzip modules */

#define LIBGZIP_VERSION "2.4"

#include <stdio.h>
#if !defined(NO_STRING_H) || defined(STDC_HEADERS)
#  include <string.h>
#  if !defined(STDC_HEADERS) && !defined(NO_MEMORY_H) && !defined(__GNUC__)
#    include <memory.h>
#  endif
#  define memzero(s, n)     memset ((s), 0, (n))
#else
#  include <strings.h>
#  define memcpy(d, s, n)   bcopy((s), (d), (n)) 
#  define memcmp(s1, s2, n) bcmp((s1), (s2), (n)) 
#  define memzero(s, n)     bzero((s), (n))
#endif

#define INBUF_EXTRA  64
#define OUTBUF_EXTRA 2048

#ifdef SMALL_MEM
#  define INBUFSIZ  8192    /* input buffer size */
#else
#  define INBUFSIZ  32768  /* input buffer size */
#endif

#ifdef SMALL_MEM
#  define OUTBUFSIZ   8192  /* output buffer size */
#else
#  define OUTBUFSIZ  16384  /* output buffer size */
#endif

#define STORED_BLOCK 0
#define STATIC_TREES 1
#define DYN_TREES    2
/* The three kinds of block type */

typedef unsigned char  uch;
typedef unsigned short ush;
typedef unsigned long  ulg;

#define WSIZE 32768	/* window size--must be a power of two, and */
			/*  at least 32K for zip's deflate method */

/* Diagnostic functions */
#ifdef DEBUG
#  define Trace(x) fprintf x
#  define Tracev(x) {fprintf x ;}
#  define Tracevv(x) {fprintf x ;}
#  define Tracec(c,x) {fprintf x ;}
#  define Tracecv(c,x) {fprintf x ;}
#else
#  define Trace(x)
#  define Tracev(x)
#  define Tracevv(x)
#  define Tracec(c,x)
#  define Tracecv(c,x)
#endif

#define near

typedef struct _InflateHandler* InflateHandler;
typedef struct _DeflateHandler* DeflateHandler;

/*****************************************************************/
/*                      INTERFACE FUNCTIONS                      */
/*****************************************************************/

/* in deflate.c */
DeflateHandler open_deflate_handler(
	long (* read_func)(char* buf, long size, void* user_val),
	void* user_val,
	int compression_level);

long deflate(DeflateHandler encoder,
	     char* decode_buff,
	     long decode_buff_size);

void close_deflate_handler(DeflateHandler encoder);


/* in inflate.c */
InflateHandler open_inflate_handler(
	long (* read_func)(char* buf, long size, void* user_val),
	void* user_val);

long inflate(InflateHandler decoder,
	     char* decode_buff,
	     long decode_buff_size);

void close_inflate_handler(InflateHandler decoder);

#endif /* ___GZIP_H_ */
