#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "gzip.h"


/* Note: gzip file format

The pkzip format imposes a lot of overhead in various headers, which
are useful for an archiver but not necessary when only one file is
compressed. gzip uses a much simpler structure. Numbers are in little
endian format, and bit 0 is the least significant bit.
A gzip file is a sequence of compressed members. Each member has the
following structure:

2 bytes  magic header  0x1f, 0x8b (\037 \213)  
1 byte   compression method
            0: store (copied)
            1: compress
            2: pack
            3: lzh
            4..7: reserved
            8: deflate
1 byte   flags
            bit 0 set: file probably ascii text
            bit 1 set: continuation of multi-part gzip file
            bit 2 set: extra field present
            bit 3 set: original file name present
            bit 4 set: file comment present
            bit 5 set: file is encrypted
            bit 6,7:   reserved
4 bytes  file modification time in Unix format
1 byte   extra flags (depend on compression method)
1 byte   operating system on which compression took place
2 bytes  optional part number (second part=1)
2 bytes  optional extra field length
? bytes  optional extra field
? bytes  optional original file name, zero terminated
? bytes  optional file comment, zero terminated
12 bytes optional encryption header
? bytes  compressed data
4 bytes  crc32
4 bytes  uncompressed input size modulo 2^32
 */

#if defined(GZIP)
/*
 * void gzip(read_function, write_function, complession_level);
 */
static void test_gzip(level)
int level;
{
    char buff[1013];
    long n;
    DeflateHandler encoder;
    encoder = open_deflate_handler(NULL, NULL, level);
    while((n = deflate(encoder, buff, sizeof(buff))) > 0)
      write(1, buff, n);
    if(n < 0)
	fprintf(stderr, "Compression error\n");
    close_deflate_handler(encoder);
}

/*
 * void gunzip(read_function, write_function);
 */
static void test_gunzip()
{
    char buff[10101];
    long n;
    InflateHandler decoder;

    decoder = open_inflate_handler(NULL, NULL);
    while((n = inflate(decoder, buff, sizeof(buff))) > 0)
	write(1, buff, n);
    if(n < 0)
	fprintf(stderr, "Decompression error\n");
    close_inflate_handler(decoder);
}

void main(int argc, char** argv)
{
    if(argc == 2 && strcmp(argv[1], "-d") == 0)
	test_gunzip();
    else
	test_gzip(6);
    exit(0);
}
#endif
